/*
 * PROJECT:     P-Test
 * LICENSE:     EUPL - https://joinup.ec.europa.eu/software/page/eupl
 * PURPOSE:     RC Supplement Library cache performance test
 * PROGRAMMERS: Ștefan Fulea (stefan.fulea@mail.md)
 */

#include <WinDef.h>
#include <WinBase.h>
#include <WinCon.h>
#include <WinUser.h>
#include <stdlib.h>

#include "p-test.h"
#include "rcsuplib.h"

#include <conio.h>

#define CHAR_RESOURCE_TEST
//#define WCHAR_RESOURCE_TEST
//#define WORD_RESOURCE_TEST
//#define DWORD_RESOURCE_TEST

INT wmain (INT argc, PWCHAR argvW[])
{
    PWCHAR pwsLoadedStr;
    WORD wLowBound, wHighLimit;
    DWORD dwReferenceVectorSize;
    PVOID * ppResults;
    HANDLE hHeap;
    PVOID pCachedVolume;

    LARGE_INTEGER objFrequency;
    LARGE_INTEGER objInitial, objFinal;
    LONGLONG llElapsedOnDirect;
    LONGLONG llElapsedOnCached;
    WORD wCounter;

    switch(argc)
    {
        case 3:
            wLowBound = _wtoi(argvW[1]);
            wHighLimit = _wtoi(argvW[2]);

            if (wHighLimit <= wLowBound)
            {
                /* report inappropriate parameters */
                return -1;
            }

            dwReferenceVectorSize =
                (wHighLimit - wLowBound) * sizeof(PWCHAR);

            hHeap = GetProcessHeap();
            if (hHeap == NULL)
            {
                /* report the heap error */
                return -1;
            }

            ppResults = HeapAlloc(hHeap, 0, dwReferenceVectorSize);

            if (ppResults == NULL)
            {
                /* report the memory allocation error */
                return -1;
            }

            SecureZeroMemory(ppResults, dwReferenceVectorSize);

            QueryPerformanceFrequency(&objFrequency);

#ifdef CHAR_RESOURCE_TEST
            /* benchmark the direct retreival of NDR content */
            QueryPerformanceCounter(&objInitial);
            for (wCounter = wLowBound; wCounter < wHighLimit; wCounter++)
                RCS_LoadStringA(NULL, PLURALFORMS_STRSET_ID,
                                (PCHAR) &ppResults[wCounter - wLowBound],
                                0,
                                wCounter);
            QueryPerformanceCounter(&objFinal);
            llElapsedOnDirect = objFinal.QuadPart - objInitial.QuadPart;

            /*for (wCounter = wLowBound; wCounter < wHighLimit; wCounter++)
            {
                _cprintf(ppResults[wCounter - wLowBound], wCounter);
                _cwprintf(L"\n");
            }*/


            /* benchmark the retreival of cached NDR */
            pCachedVolume = RCS_CacheStrSetA(NULL, PLURALFORMS_STRSET_ID);
            if (pCachedVolume == NULL)
                return -1;
            QueryPerformanceCounter(&objInitial);
            for (wCounter = wLowBound; wCounter < wHighLimit; wCounter++)
                ppResults[wCounter - wLowBound] =
                    (PVOID) RCS_GetStringA(pCachedVolume, wCounter);
            QueryPerformanceCounter(&objFinal);
            llElapsedOnCached = objFinal.QuadPart - objInitial.QuadPart;

            /*for (wCounter = wLowBound; wCounter < wHighLimit; wCounter++)
            {
                _cprintf(ppResults[wCounter - wLowBound], wCounter);
                _cwprintf(L"\n");
            }*/

            RCS_FreeCachedResourceSet(pCachedVolume);
#endif

#ifdef WCHAR_RESOURCE_TEST
            /* benchmark the direct retreival of NDR content */
            QueryPerformanceCounter(&objInitial);
            for (wCounter = wLowBound; wCounter < wHighLimit; wCounter++)
                RCS_LoadStringW(NULL, PLURALFORMS_WSTRSET_ID,
                                (PWCHAR) &ppResults[wCounter - wLowBound],
                                0,
                                wCounter);
            QueryPerformanceCounter(&objFinal);
            llElapsedOnDirect = objFinal.QuadPart - objInitial.QuadPart;

            /*for (wCounter = wLowBound; wCounter < wHighLimit; wCounter++)
            {
                _cwprintf(ppResults[wCounter - wLowBound], wCounter);
                _cwprintf(L"\n");
            }*/


            /* benchmark the retreival of cached NDR */
            pCachedVolume = RCS_CacheStrSetW(NULL, PLURALFORMS_WSTRSET_ID);
            if (pCachedVolume == NULL)
                return -1;
            QueryPerformanceCounter(&objInitial);
            for (wCounter = wLowBound; wCounter < wHighLimit; wCounter++)
                ppResults[wCounter - wLowBound] =
                    (PVOID) RCS_GetStringW(pCachedVolume, wCounter);
            QueryPerformanceCounter(&objFinal);
            llElapsedOnCached = objFinal.QuadPart - objInitial.QuadPart;

            /*for (wCounter = wLowBound; wCounter < wHighLimit; wCounter++)
            {
                _cwprintf(ppResults[wCounter - wLowBound], wCounter);
                _cwprintf(L"\n");
            }*/

            RCS_FreeCachedResourceSet(pCachedVolume);
#endif

#ifdef WORD_RESOURCE_TEST
            /* benchmark the direct retreival of NDR content */
            QueryPerformanceCounter(&objInitial);
            for (wCounter = wLowBound; wCounter < wHighLimit; wCounter++)
                RCS_LoadWORD(NULL, PLURALFORMS_WORDSET_ID,
                             (PWORD) &ppResults[wCounter - wLowBound],
                             wCounter);
            QueryPerformanceCounter(&objFinal);
            llElapsedOnDirect = objFinal.QuadPart - objInitial.QuadPart;

            /*for (wCounter = wLowBound; wCounter < wHighLimit; wCounter++)
                _cwprintf(L"%d:%d\n",
                          wCounter,
                          ((PWORD)ppResults)[wCounter - wLowBound]);*/


            /* benchmark the retreival of cached NDR */
            pCachedVolume = RCS_CacheWORDSet(NULL, PLURALFORMS_WORDSET_ID);
            if (pCachedVolume == NULL)
                return -1;
            QueryPerformanceCounter(&objInitial);
            for (wCounter = wLowBound; wCounter < wHighLimit; wCounter++)
                ((PWORD)ppResults)[wCounter - wLowBound] =
                    RCS_GetWORD(pCachedVolume, wCounter);
            QueryPerformanceCounter(&objFinal);
            llElapsedOnCached = objFinal.QuadPart - objInitial.QuadPart;

            /*for (wCounter = wLowBound; wCounter < wHighLimit; wCounter++)
                _cwprintf(L"%d:%d\n",
                          wCounter,
                          ((PWORD)ppResults)[wCounter - wLowBound]);*/

            RCS_FreeCachedResourceSet(pCachedVolume);
#endif

#ifdef DWORD_RESOURCE_TEST
            /* benchmark the direct retreival of NDR content */
            QueryPerformanceCounter(&objInitial);
            for (wCounter = wLowBound; wCounter < wHighLimit; wCounter++)
                RCS_LoadDWORD(NULL, PLURALFORMS_DWORDSET_ID,
                              (PDWORD) &ppResults[wCounter - wLowBound],
                              wCounter);
            QueryPerformanceCounter(&objFinal);
            llElapsedOnDirect = objFinal.QuadPart - objInitial.QuadPart;

            /*for (wCounter = wLowBound; wCounter < wHighLimit; wCounter++)
                _cwprintf(L"%d:%d\n",
                          wCounter,
                          ((PDWORD)ppResults)[wCounter - wLowBound]);*/


            /* benchmark the retreival of cached NDR */
            pCachedVolume = RCS_CacheDWORDSet(NULL, PLURALFORMS_DWORDSET_ID);
            if (pCachedVolume == NULL)
                return -1;
            QueryPerformanceCounter(&objInitial);
            for (wCounter = wLowBound; wCounter < wHighLimit; wCounter++)
                ((PDWORD)ppResults)[wCounter - wLowBound] =
                    RCS_GetDWORD(pCachedVolume, wCounter);
            QueryPerformanceCounter(&objFinal);
            llElapsedOnCached = objFinal.QuadPart - objInitial.QuadPart;

            /*for (wCounter = wLowBound; wCounter < wHighLimit; wCounter++)
                _cwprintf(L"%d:%d\n",
                          wCounter,
                          ((PDWORD)ppResults)[wCounter - wLowBound]);*/

            RCS_FreeCachedResourceSet(pCachedVolume);
#endif

            LoadStringW(NULL, REPORT_WSTR_ID, (PWCHAR) &pwsLoadedStr, 0);
            _cwprintf(pwsLoadedStr,
                      wHighLimit - wLowBound,
                      (llElapsedOnDirect * 1000.0) / objFrequency.QuadPart,
                      (llElapsedOnCached * 1000.0) / objFrequency.QuadPart);

            HeapFree(hHeap, 0, ppResults);
            break;

        default:
            LoadStringW(NULL, HELP_WSTR_ID, (PWCHAR) &pwsLoadedStr, 0);
            _cwprintf(pwsLoadedStr);
    };

    return 0;
}

/* EOF */
